#include "stdafx.h"
#include "game.h"
#include "clf_geometry.h"

extern HWND  hWnd;						// Window Handle
extern bool keyDown[256];			// holds true / false for each key

float anglex=0.f;
float angley=0.f;
float anglez=0.f;

class cCamera{
public:
	clf_vector3D m_pos;
	clf_vector3D m_look;
	clf_vector3D m_up;

	cCamera(){
		// setup a default viewer
		setup(clf_vector3D(0,0,3),        // position viewer
					clf_vector3D(0,0,0),        // setup the point of interest (where we are looking at)
					clf_vector3D(0,1,0));       // which way is up
	}
	~cCamera(){}
	void setup(clf_vector3D& pos, clf_vector3D& lookat, clf_vector3D& up){
		m_pos = pos;
		m_look = lookat;
		m_up = up;
	}
	void move_to(clf_vector3D& pos){
		m_pos = pos;
	}
	void look_at(clf_vector3D& lookat){
		m_look = lookat;
	}
	void orient(clf_vector3D& up){
		m_up = up;
	}
	void apply(){
		// the following is 
		gluLookAt(m_pos.x, m_pos.y, m_pos.z,     // viewer position
					m_look.x, m_look.y, m_look.z,      // focus point
					m_up.x, m_up.y, m_up.z);           // up direction
	}
};

class clf_stage{
public:
	clf_stage(){
		left = bottom = 0;
		near_plane = 1.f;
		far_plane = 100.f;
		perspective_angle = 30;
	}
	~clf_stage(){
	}

	void resize(int l, int b, int w, int h){
		left = l;
		bottom = b;
		width = w;
		height = h;
	}

	void set_perspective(float angle, float np, float fp){
		perspective_angle = angle;
		near_plane = np;
		far_plane = fp;
	}

	void prepare(){
		glViewport (left, bottom, (GLsizei)(width), (GLsizei)(height));				// reset the current viewport
		glMatrixMode (GL_PROJECTION);										// select the projection matrix
		glLoadIdentity ();													// reset the projection matrix

		float w = (float)width;
		float h = (float)height;

		// the two paths are exactly the same (internally)
		// in the first we calculate the dimensions of the near plane visible area
		// which requires some elementary but sometimes forgotten trigonometry
		// the second is provided by the utility functions in OpenGL
#if 1
		float r = w / h;
		float persp_angle = perspective_angle/2.f;      // half of the perspective angle goes to one side
		float ys = near_plane*(float)tan(persp_angle*PI/180.f);
		float xs = ys*r;
		glFrustum( -xs, xs, -ys, ys, near_plane, far_plane );
#else
		gluPerspective (perspective_angle, w/h, near_plane, far_plane);	// calculate the aspect ratio of the window
#endif
	}

	int left, bottom;
	int width, height;
	float near_plane, far_plane;
	float perspective_angle;
};

cCamera g_cam;
cCamera g_cam2;
clf_stage g_stage;
clf_stage g_stage2;
void resize_window (int width, int height)								// reshape the window when it's moved or resized
{
#if 0
	// use the whole window
	g_stage.resize(0,0,width, height);
	g_stage2.resize(0,0,width, height);
#else
	// or use an area half the window size and centered in it
	g_stage.resize(0,height/4,width/2, height/2);
	g_stage2.resize(width/2,height/4,width/2, height/2);
#endif
}

void init_game()
{
	g_cam.setup(clf_vector3D(0,0,15),       // position viewer
							clf_vector3D(0,0,0),        // setup the point of interest (where we are looking at)
							clf_vector3D(0,1,0));       // which way is up
	g_cam2.setup(clf_vector3D(15,0,15),       // position viewer
							clf_vector3D(0,0,0),        // setup the point of interest (where we are looking at)
							clf_vector3D(0,1,0));       // which way is up
}

void frame_move(float fElapsed)
{
	if (keyDown[VK_ESCAPE]) 
		DestroyWindow(hWnd);

	float amount = fElapsed * 45.f; // 45 degrees / second

	if (keyDown[VK_END]) 
		anglez -= amount;
	if (keyDown[VK_HOME]) 
		anglez += amount;
	if (keyDown[VK_LEFT])
		angley -= amount;
	if (keyDown[VK_RIGHT]) 
		angley += amount;
	if (keyDown[VK_UP]) 
		anglex -= amount;
	if (keyDown[VK_DOWN]) 
		anglex += amount;
}

static void draw_reference_cube()
{
	// the reference cube is always drawn in local coordinates!!!
	float span = 2.5f;
	glBegin(GL_LINES);
		// draw the axes
		glColor3f(1.f, 0.f, 0.f);
		glVertex3f( -span/4, 0.f, 0.f); glVertex3f( span, 0.f, 0.f);
		glColor3f(0.f, 1.f, 0.f);
		glVertex3f( 0.f, -span/4, 0.f); glVertex3f( 0.f, span, 0.f);
		glColor3f(0.f, 0.f, 1.f);
		glVertex3f( 0.f, 0.f, -span/4); glVertex3f( 0.f, 0.f, span);

		// draw the colored diagonals
		glColor3f(1.f, 0.f, 1.f);
		glVertex3f( 0.f, 0.f, 0); glVertex3f( span, 0, span);
		glColor3f(0.f, 1.f, 1.f);
		glVertex3f( 0.f, 0.f, 0); glVertex3f( 0, span, span);
		glColor3f(1.f, 1.f, 0.f);
		glVertex3f( 0.f, 0.f, 0); glVertex3f( span, span, 0);

		// draw the rest of the edges of the reference cube
		glColor3f(1.f, 1.f, 1.f);
		glVertex3f( 0.f, 0, span); glVertex3f( span, 0, span);
		glVertex3f( span, 0, span); glVertex3f( span, 0, 0);

		glVertex3f( 0.f, span, span); glVertex3f( span, span, span);
		glVertex3f( span, span, span); glVertex3f( span, span, 0);
		glVertex3f( span, span, 0); glVertex3f( 0, span, 0);
		glVertex3f( 0, span, 0); glVertex3f( 0, span, span);

		glVertex3f( span, 0, span); glVertex3f( span, span, span);
		glVertex3f( 0, 0, span); glVertex3f( 0, span, span);
		glVertex3f( span, 0, 0); glVertex3f( span, span, 0);
	glEnd();
}

static void draw_moved_cube()
{
	// the reference cube is always drawn in local coordinates!!!
	float span = 1.25f;
	float xpos = 3.f;
	float ypos = -2.f;
	float zpos = -3.f;

	glBegin(GL_LINES);
		// draw the axes
		glColor3f(1.f, 0.f, 0.f);
		glVertex3f( xpos-span/4, ypos+0.f, zpos+0.f); glVertex3f( xpos+span, ypos+0.f, zpos+0.f);
		glColor3f(0.f, 1.f, 0.f);
		glVertex3f( xpos+0.f, ypos-span/4, zpos+0.f); glVertex3f( xpos+0.f, ypos+span, zpos+0.f);
		glColor3f(0.f, 0.f, 1.f);
		glVertex3f( xpos+0.f, ypos+0.f, zpos+span/4); glVertex3f( xpos+0.f, ypos+0.f, zpos+span);

		// draw the colored diagonals
		glColor3f(1.f, 0.f, 1.f);
		glVertex3f( xpos+0.f, ypos+0.f, zpos+0); glVertex3f( xpos+span, ypos+0, zpos+span);
		glColor3f(0.f, 1.f, 1.f);
		glVertex3f( xpos+0.f, ypos+0.f, zpos+0); glVertex3f( xpos+0, ypos+span, zpos+span);
		glColor3f(1.f, 1.f, 0.f);
		glVertex3f( xpos+0.f, ypos+0.f, zpos+0); glVertex3f( xpos+span, ypos+span, zpos+0);

		// draw the rest of the edges of the reference cube
		glColor3f(1.f, 1.f, 1.f);
		glVertex3f( xpos+0.f, ypos+0, zpos+span); glVertex3f( xpos+span, ypos+0, zpos+span);
		glVertex3f( xpos+span, ypos+0, zpos+span); glVertex3f( xpos+span, ypos+0, zpos+0);

		glVertex3f( xpos+0.f, ypos+span, zpos+span); glVertex3f( xpos+span, ypos+span, zpos+span);
		glVertex3f( xpos+span, ypos+span, zpos+span); glVertex3f( xpos+span, ypos+span, zpos+0);
		glVertex3f( xpos+span, ypos+span, zpos+0); glVertex3f( xpos+0, ypos+span, zpos+0);
		glVertex3f( xpos+0, ypos+span, zpos+0); glVertex3f( xpos+0, ypos+span, zpos+span);

		glVertex3f( xpos+span, ypos+0, zpos+span); glVertex3f( xpos+span, ypos+span, zpos+span);
		glVertex3f( xpos+0, ypos+0, zpos+span); glVertex3f( xpos+0, ypos+span, zpos+span);
		glVertex3f( xpos+span, ypos+0, zpos+0); glVertex3f( xpos+span, ypos+span, zpos+0);
	glEnd();
}

void draw_scene()
{
	// whatever coordinates come from this point on are model coordinates
	glMatrixMode(GL_MODELVIEW);
	// initialize the transformations
	glLoadIdentity();

	// first rotate the cube
	glPushMatrix();			 // save state
		glRotatef(anglex, 1,0,0);     // apply transformation
		glRotatef(angley, 0,1,0);
		glRotatef(anglez, 0,0,1);
		draw_reference_cube();        // draw
	glPopMatrix();       // restore state

	// now position the 'object' somewhere else
	glPushMatrix();      // save state
		// scaling before moving will scale the movement as well
		glTranslatef(-4,-2,0);         // move the coordinate system
		glScalef(.15f, .25f, .35f);    // set the scaling
		draw_reference_cube();         // draw
	glPopMatrix();       // restore state

	// now draw an 'object' that is already somewhere else
	glPushMatrix();      // save state
		draw_moved_cube();             // draw
	glPopMatrix();       // restore state

	// and now move and scale it
	glPushMatrix();      // save state
		glTranslatef(-4,-2,0);         // move the coordinate system
		glScalef(.5f, .5f, .5f);       // set the scaling
		// notice that scaling affects the location coordinates as well
		// if you want to scale things and still have control over their position
		// you should prefer the first method, use objects that draw themselves
		// in local coordinates as the reference cube
		draw_moved_cube();             // draw
	glPopMatrix();       // restore state
}

void frame_render()
{
	// clear the screen
	glClearColor (0.0f, 0.0f, .1f, 1.f);				// black background
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// clear screen and depth buffer

	// prepare the stage
	g_stage.prepare();
	// enable depth test
	glEnable (GL_DEPTH_TEST);
	glDepthFunc (GL_LESS);		 // visible is the one closer to the viewer
	// position the viewer
	g_cam.apply();
	draw_scene();
	
	// prepare the stage
	g_stage2.prepare();
	// enable depth test
	glEnable (GL_DEPTH_TEST);
	glDepthFunc (GL_LESS);		 // visible is the one closer to the viewer
	// position the viewer
	g_cam2.apply();
	draw_scene();
}
