#include "stdafx.h"
#include "start_3d.h"
#include "clf_timer.h"
#include "game.h"

static TCHAR szTitle[100];					// The title bar text
static TCHAR szWindowClass[100];			// the main window class name

static HINSTANCE hInst;						// current instance
HWND  hWnd=0;						// Window Handle
static HDC   hDC=0;							// Device Context
static HGLRC hRC=0;							// Rendering Context
static bool isMinimized = FALSE;			// is window visible or minimized

// windows stuff
static BOOL InitInstance(HINSTANCE, int);
static LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);

// our stuff
static HWND create_GL_window (int wid, int hei, int bitsPerPixel, const char* title, HINSTANCE hInstance, const char* classname, int stencilBuffer);
static bool destroy_GL_window ();


bool keyDown [256];			// holds true / false for each key

// program entry point
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
	UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(lpCmdLine);

	MSG msg;

	strcpy(szTitle, "GusOnGames");
	strcpy(szWindowClass, "GusOnGames");

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow))
	{
		return FALSE;
	}

	// zero the keystatus buffers
	memset(keyDown, 0, sizeof(keyDown));

	init_game();
	// start the timer before we enter the main loop
	get_global_timer()->start();
	// Main message loop:
	bool bLooping = true;
	while (bLooping)
	{
		// check for windows message and precess them
		if (PeekMessage (&msg, NULL, 0, 0, PM_REMOVE) != 0)
		{
			if (msg.message == WM_QUIT)
				bLooping = false;

			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else  // no messages, just loop for next frame
		{
			if (isMinimized)		// if window is minimized
			{
				WaitMessage ();		// yeld back to system, do not waste processing power
			}
			else
			{
				// update the scene based on the elapsed time since last update
				frame_move((float)get_global_timer()->get_elapsed_time());
				frame_render();		// render scene
				SwapBuffers (hDC);	// Swap Buffers (Double Buffering)
			}
		}
	}
	destroy_GL_window();
	// stop the clock
	get_global_timer()->stop();

	return (int) msg.wParam;
}

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	HWND hWnd;

	hInst = hInstance; // Store instance handle in our global variable

	WNDCLASSEX wcex;
	ZeroMemory (&wcex, sizeof (WNDCLASSEX));

	wcex.cbSize = sizeof(WNDCLASSEX);

	wcex.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wcex.lpfnWndProc	= WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_SIMPLE_APP));
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_APPWORKSPACE);
	wcex.lpszMenuName	= MAKEINTRESOURCE(IDC_SIMPLE_APP);
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, MAKEINTRESOURCE(IDI_SMALL));

	if (RegisterClassEx (&wcex) == 0)
	{
		MessageBox (HWND_DESKTOP, "RegisterClassEx Failed!", "Error", MB_OK | MB_ICONEXCLAMATION);
		return false;
	}

	hWnd = create_GL_window (800, 600, 32, szTitle, hInstance, szWindowClass, 1);

	if (!hWnd)
		return FALSE;

	return TRUE;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	PAINTSTRUCT ps;
	HDC hdc;

	switch (message)
	{
	case WM_COMMAND: // menu commands
		wmId    = LOWORD(wParam);
		wmEvent = HIWORD(wParam);
		switch (wmId)
		{
		case IDM_EXIT:
			DestroyWindow(hWnd);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
		}
		break;
	case WM_SIZE:	// window size change
		switch (wParam)	// evaluate
		{
		case SIZE_MINIMIZED:	// minimized?
			isMinimized = true;	// set isMinimized to true
			return 0;

		case SIZE_MAXIMIZED:	// maximized?
			isMinimized = false;// set isMinimized to false
			resize_window (LOWORD (lParam), HIWORD (lParam));	// recalc view - LoWord=Width, HiWord=Height
			return 0;

		case SIZE_RESTORED:		// restored?
			isMinimized = false;// set isMinimized to false
			resize_window (LOWORD (lParam), HIWORD (lParam));	// recalc view - LoWord=Width, HiWord=Height
			return 0;
		}
		break;
	case WM_KEYDOWN:												// Update Keyboard Buffers For clf_keys Pressed
		if ((wParam >= 0) && (wParam <= 255))						// Is Key (wParam) In A Valid Range?
		{
			keyDown [wParam] = true;					// Set The Selected Key (wParam) To True
			return 0;												// Return
		}
		break;															// Break

	case WM_KEYUP:													// Update Keyboard Buffers For clf_keys Released
		if ((wParam >= 0) && (wParam <= 255))						// Is Key (wParam) In A Valid Range?
		{
			keyDown [wParam] = false;					// Set The Selected Key (wParam) To False
			return 0;												// Return
		}
		break;															// Break

	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);
		EndPaint(hWnd, &ps);
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

// This function Creates Our OpenGL Window
HWND create_GL_window (int wid, int hei, int bitsPerPixel, const char* title, HINSTANCE hInstance, const char* classname, int stencilBuffer)
{
	DWORD windowStyle = WS_OVERLAPPEDWINDOW;				// define our window style
	DWORD windowExtendedStyle = WS_EX_APPWINDOW;			// define the window's extended style

	int width = wid;
	int height = hei;

	RECT windowRect = {0, 0, width, height};				// define our window coordinates

	// adjust window, account for window borders
	AdjustWindowRectEx (&windowRect, windowStyle, 0, windowExtendedStyle);

	// create the opengl window
	hWnd = CreateWindowEx (windowExtendedStyle,	// extended style
		classname,								// class name
		title,									// window title
		windowStyle,							// window style
		0, 0,									// window x,y position
		windowRect.right - windowRect.left,		// window width
		windowRect.bottom - windowRect.top,		// window height
		HWND_DESKTOP,							// desktop is window's parent
		0,										// no menu
		hInstance,								// pass the window instance
		0);

	if (hWnd == 0)								// was window creation a success?
	{
		return hWnd;							// if not return false
	}

	hDC = GetDC(hWnd);							// grab a device context for this window
	if (hDC == 0)								// did we get a device context?
	{
		// Failed
		DestroyWindow (hWnd);					// destroy the window
		hWnd = 0;								// zero the window handle
		return hWnd;							// return false
	}

	PIXELFORMATDESCRIPTOR pfd =								// pfd tells windows how we want things to be
	{
		sizeof (PIXELFORMATDESCRIPTOR),						// size of this pixel format descriptor
		1,													// version number
		PFD_DRAW_TO_WINDOW |								// format must support window
		PFD_SUPPORT_OPENGL |								// format must support opengl
		PFD_DOUBLEBUFFER,									// must support double buffering
		PFD_TYPE_RGBA,										// request an rgba format
		bitsPerPixel,										// select our color depth
		0, 0, 0, 0, 0, 0,									// color bits ignored
		0,													// no alpha buffer
		0,													// shift bit ignored
		0,													// no accumulation buffer
		0, 0, 0, 0,											// accumulation bits ignored
		16,													// 16bit z-buffer (depth buffer)  
		stencilBuffer,										// stencil buffer
		0,													// no auxiliary buffer
		PFD_MAIN_PLANE,										// main drawing layer
		0,													// reserved
		0, 0, 0												// layer masks ignored
	};

	GLuint PixelFormat = ChoosePixelFormat (hDC, &pfd);	// find a compatible pixel format
	if (PixelFormat == 0)								// did we find a compatible format?
	{
		// Failed
		ReleaseDC (hWnd, hDC);							// release our device context
		hDC = 0;										// zero the device context
		DestroyWindow (hWnd);							// destroy the window
		hWnd = 0;										// zero the window handle
		return hWnd;									// return false
	}

	if (SetPixelFormat (hDC, PixelFormat, &pfd) == false)// try to set the pixel format
	{
		// Failed
		ReleaseDC (hWnd, hDC);							// release our device context
		hDC = 0;										// zero the device context
		DestroyWindow (hWnd);							// destroy the window
		hWnd = 0;										// zero the window handle
		return hWnd;									// return false
	}

	hRC = wglCreateContext (hDC);						// try to get a rendering context
	if (hRC == 0)										// did we get a rendering context?
	{
		// Failed
		ReleaseDC (hWnd, hDC);							// release our device context
		hDC = 0;										// zero the device context
		DestroyWindow (hWnd);							// destroy the window
		hWnd = 0;										// zero the window handle
		return hWnd;									// return false
	}

	// make the rendering context our current rendering context
	if (wglMakeCurrent (hDC, hRC) == false)				// failed
	{
		wglDeleteContext (hRC);							// delete the rendering context
		hRC = 0;										// zero the rendering context
		ReleaseDC (hWnd, hDC);							// release our device context
		hDC = 0;										// zero the device context
		DestroyWindow (hWnd);							// destroy the window
		hWnd = 0;										// zero the window handle
		return hWnd;									// return false
	}

	ShowWindow (hWnd, SW_NORMAL);						// make the window visible
	isMinimized = false;								// set isMinimized to false

	resize_window (width, height);						// reshape our gl window
	UpdateWindow(hWnd);

	return hWnd;										// window creating was a success
}

bool destroy_GL_window ()								// destroy the opengl window & release resources
{
	if (hWnd != 0)										// does the window have a handle?
	{	
		if (hDC != 0)									// does the window have a device context?
		{
			wglMakeCurrent (hDC, 0);					// set the current active rendering context to zero
			if (hRC != 0)								// does the window have a rendering context?
			{
				wglDeleteContext (hRC);					// release the rendering context
				hRC = 0;								// zero the rendering context

			}
			ReleaseDC (hWnd, hDC);						// release the device context
			hDC = 0;									// zero the device context
		}
		DestroyWindow (hWnd);							// destroy the window
		hWnd = 0;										// zero the window handle
	}

	return true;										// return true
}

/////////////////////////////////////////////////////////////////////////////////////////////////////
